@@ -35,6 +35,7 @@ Example::
 import xml.etree.ElementTree as XML
 import jenkins_jobs.modules.base
 from jenkins_jobs.errors import JenkinsJobsException
+import logging
 
 
 def builds_chain_fingerprinter(parser, xml_parent, data):
@@ -346,78 +347,19 @@ def authorization(parser, xml_parent, data):
 
 def extended_choice(parser, xml_parent, data):
     """yaml: extended-choice
-    Creates an extended choice property where values can be read from a file
-    Requires the Jenkins `Extended Choice Parameter Plugin.
-    <https://wiki.jenkins-ci.org/display/JENKINS/
-    Extended+Choice+Parameter+plugin>`_
-
-    :arg string name: name of the property
-    :arg string description: description of the property (optional, default '')
-    :arg string property-file: location of property file to read from
-        (optional, default '')
-    :arg string property-key: key for the property-file (optional, default '')
-    :arg bool quote-value: whether to put quotes around the property
-        when passing to Jenkins (optional, default false)
-    :arg string visible-items: number of items to show in the list
-        (optional, default 5)
-    :arg string type: type of select (optional, default single-select)
-    :arg string value: comma separated list of values for the single select
-        or multi-select box (optional, default '')
-    :arg string default-value: used to set the initial selection of the
-        single-select or multi-select box (optional, default '')
-    :arg string default-property-file: location of property file when default
-        value needs to come from a property file (optional, default '')
-    :arg string default-property-key: key for the default property file
-        (optional, default '')
-
-    Example::
-
-      properties:
-        - extended-choice:
-            name: FOO
-            description: A foo property
-            property-file: /home/foo/property.prop
-            property-key: key
-            quote-value: true
-            visible-items: 10
-            type: multi-select
-            value: foo,bar,select
-            default-value: foo
-            default-property-file: /home/property.prop
-            default-property-key: fookey
+    Use of this config option is deprecated.  You should use the
+    `extended-choice` option in the parameter section of the job configuration
+    instead.
     """
+    logger = logging.getLogger("%s:extended_choice" % __name__)
+    logger.warn('Use of the extended-choice property is deprecated.  You '
+                'should use the extended-choice option in the parameter '
+                'section instead.')
     definition = XML.SubElement(xml_parent,
                                 'hudson.model.ParametersDefinitionProperty')
     definitions = XML.SubElement(definition, 'parameterDefinitions')
-    extended = XML.SubElement(definitions, 'com.cwctravel.hudson.plugins.'
-                                           'extended__choice__parameter.'
-                                           'ExtendedChoiceParameterDefinition')
-    XML.SubElement(extended, 'name').text = data['name']
-    XML.SubElement(extended, 'description').text = data.get('description', '')
-    XML.SubElement(extended, 'quoteValue').text = str(data.get('quote-value',
-                                                      False)).lower()
-    XML.SubElement(extended, 'visibleItemCount').text = data.get(
-        'visible-items', '5')
-    choice = data.get('type', 'single-select')
-    choicedict = {'single-select': 'PT_SINGLE_SELECT',
-                  'multi-select': 'PT_MULTI_SELECT',
-                  'radio': 'PT_RADIO',
-                  'checkbox': 'PT_CHECKBOX'}
-    if choice not in choicedict:
-        raise JenkinsJobsException("Type entered is not valid, must be one "
-                                   "of: single-select, multi-select, radio, "
-                                   "or checkbox")
-    XML.SubElement(extended, 'type').text = choicedict[choice]
-    XML.SubElement(extended, 'value').text = data.get('value', '')
-    XML.SubElement(extended, 'propertyFile').text = data.get('property-file',
-                                                             '')
-    XML.SubElement(extended, 'propertyKey').text = data.get('property-key', '')
-    XML.SubElement(extended, 'defaultValue').text = data.get('default-value',
-                                                             '')
-    XML.SubElement(extended, 'defaultPropertyFile').text = data.get(
-        'default-property-file', '')
-    XML.SubElement(extended, 'defaultPropertyKey').text = data.get(
-        'default-property-key', '')
+    parser.registry.dispatch('parameter', parser, definitions,
+                             {'extended-choice': data})
 
 
 def priority_sorter(parser, xml_parent, data):
