@@ -96,7 +96,17 @@ remoteName/\*')
       (default 'default')
     :arg str git-config-name: Configure name for Git clone (optional)
     :arg str git-config-email: Configure email for Git clone (optional)
-    :arg str timeout: Timeout for git commands in minutes (optional)
+
+
+    :extensions:
+        :arg dict changelog-against:
+            :changelog-against:
+                * **remote** (`string`) - name of repo that contains branch to
+                    create changelog against (default 'origin')
+                * **branch** (`string`) - name of the branch to create
+                    create changelog against (default 'master')
+
+        :arg str timeout: Timeout for git commands in minutes (optional)
 
     :browser values:
         :auto:
@@ -161,9 +171,9 @@ remoteName/\*')
         data['remotes'] = [{data.get('name', 'origin'): data.copy()}]
     for remoteData in data['remotes']:
         huser = XML.SubElement(user, 'hudson.plugins.git.UserRemoteConfig')
-        remoteName = remoteData.keys()[0]
+        remoteName = next(iter(remoteData.keys()))
         XML.SubElement(huser, 'name').text = remoteName
-        remoteParams = remoteData.values()[0]
+        remoteParams = next(iter(remoteData.values()))
         if 'refspec' in remoteParams:
             refspec = remoteParams['refspec']
         else:
@@ -223,9 +233,17 @@ remoteName/\*')
     if 'local-branch' in data:
         XML.SubElement(scm, 'localBranch').text = data['local-branch']
 
+    exts_node = XML.SubElement(scm, 'extensions')
+    if 'changelog-against' in data:
+        ext_name = 'hudson.plugins.git.extensions.impl.ChangelogToBranch'
+        ext = XML.SubElement(exts_node, ext_name)
+        opts = XML.SubElement(ext, 'options')
+        change_remote = data['changelog-against'].get('remote', 'origin')
+        change_branch = data['changelog-against'].get('branch', 'master')
+        XML.SubElement(opts, 'compareRemote').text = change_remote
+        XML.SubElement(opts, 'compareTarget').text = change_branch
     if 'timeout' in data:
-        ext = XML.SubElement(scm, 'extensions')
-        co = XML.SubElement(ext,
+        co = XML.SubElement(exts_node,
                             'hudson.plugins.git.extensions.impl.'
                             'CheckoutOption')
         XML.SubElement(co, 'timeout').text = str(data['timeout'])
@@ -368,7 +386,7 @@ def store(parser, xml_parent, data):
     pundles = XML.SubElement(scm, 'pundles')
     for pundle_spec in pundle_specs:
         pundle = XML.SubElement(pundles, '{0}.PundleSpec'.format(namespace))
-        pundle_type = pundle_spec.keys()[0]
+        pundle_type = next(iter(pundle_spec))
         pundle_name = pundle_spec[pundle_type]
         if pundle_type not in valid_pundle_types:
             raise JenkinsJobsException(
@@ -397,6 +415,8 @@ def svn(parser, xml_parent, data):
     :arg str url: URL of the svn repository
     :arg str basedir: location relative to the workspace root to checkout to
       (default '.')
+    :arg str credentials-id: optional argument to specify the ID of credentials
+      to use
     :arg str workspaceupdater: optional argument to specify
       how to update the workspace (default wipeworkspace)
     :arg list(str) excluded-users: list of users to ignore revisions from
@@ -417,6 +437,7 @@ def svn(parser, xml_parent, data):
       :Repo: * **url** (`str`) -- URL for the repository
              * **basedir** (`str`) -- Location relative to the workspace
                                       root to checkout to (default '.')
+             * **credentials-id** - optional ID of credentials to use
 
     :workspaceupdater values:
              :wipeworkspace: - deletes the workspace before checking out
@@ -442,11 +463,17 @@ def svn(parser, xml_parent, data):
                                     'hudson.scm.SubversionSCM_-ModuleLocation')
             XML.SubElement(module, 'remote').text = repo['url']
             XML.SubElement(module, 'local').text = repo.get('basedir', '.')
+            if 'credentials-id' in repo:
+                XML.SubElement(module, 'credentialsId').text = repo[
+                    'credentials-id']
     elif 'url' in data:
         module = XML.SubElement(locations,
                                 'hudson.scm.SubversionSCM_-ModuleLocation')
         XML.SubElement(module, 'remote').text = data['url']
         XML.SubElement(module, 'local').text = data.get('basedir', '.')
+        if 'credentials-id' in data:
+            XML.SubElement(module, 'credentialsId').text = data[
+                'credentials-id']
     else:
         raise JenkinsJobsException("A top level url or repos list must exist")
     updater = data.get('workspaceupdater', 'wipeworkspace')
@@ -507,9 +534,9 @@ def tfs(parser, xml_parent, data):
         server.
     :arg str login: The user name that is registered on the server. The user
         name must contain the name and the domain name. Entered as
-        domain\\\user or user\@domain (optional).
+        domain\\\\user or user\@domain (optional).
         **NOTE**: You must enter in at least two slashes for the
-        domain\\\user format in JJB YAML. It will be rendered normally.
+        domain\\\\user format in JJB YAML. It will be rendered normally.
     :arg str use-update: If true, Hudson will not delete the workspace at end
         of each build. This causes the artifacts from the previous build to
         remain when a new build starts. (default true)
@@ -651,6 +678,106 @@ def workspace(parser, xml_parent, data):
         XML.SubElement(workspace, 'criteria').text = criteria
 
 
+def hg(self, xml_parent, data):
+    """yaml: hg
+    Specifies the mercurial SCM repository for this job.
+    Requires the Jenkins `Mercurial Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Mercurial+Plugin>`_
+
+    :arg str url: URL of the hg repository
+    :arg str credentials-id: ID of credentials to use to connect (optional)
+    :arg str revision-type: revision type to use (default 'branch')
+    :arg str revision: the branch or tag name you would like to track
+      (default 'default')
+    :arg list(str) modules: reduce unnecessary builds by specifying a list of
+      "modules" within the repository. A module is a directory name within the
+      repository that this project lives in. (default '')
+    :arg bool clean: wipe any local modifications or untracked files in the
+      repository checkout (default false)
+    :arg str subdir: check out the Mercurial repository into this
+      subdirectory of the job's workspace (optional)
+    :arg bool disable-changelog: do not calculate the Mercurial changelog
+      for each build (default false)
+    :arg str browser: what repository browser to use (default 'auto')
+    :arg str browser-url: url for the repository browser
+      (required if browser is set)
+
+    :browser values:
+        :fisheye:
+        :bitbucketweb:
+        :googlecode:
+        :hgweb:
+        :kilnhg:
+        :rhodecode:
+        :rhodecodelegacy:
+
+    Example:
+
+    .. literalinclude:: ../../tests/scm/fixtures/hg02.yaml
+    """
+    scm = XML.SubElement(xml_parent, 'scm', {'class':
+                         'hudson.plugins.mercurial.MercurialSCM'})
+    if 'url' in data:
+        XML.SubElement(scm, 'source').text = data['url']
+    else:
+        raise JenkinsJobsException("A top level url must exist")
+
+    if 'credentials-id' in data:
+        XML.SubElement(scm, 'credentialsId').text = data['credentials-id']
+
+    revision_type_dict = {
+        'branch': 'BRANCH',
+        'tag': 'TAG',
+    }
+    try:
+        revision_type = revision_type_dict[data.get('revision-type', 'branch')]
+    except KeyError:
+        raise JenkinsJobsException('Invalid revision-type %r' %
+                                   data.get('revision-type'))
+    XML.SubElement(scm, 'revisionType').text = revision_type
+
+    XML.SubElement(scm, 'revision').text = data.get('revision', 'default')
+
+    if 'subdir' in data:
+        XML.SubElement(scm, 'subdir').text = data['subdir']
+
+    xc = XML.SubElement(scm, 'clean')
+    xc.text = str(data.get('clean', False)).lower()
+
+    modules = data.get('modules', '')
+    if isinstance(modules, list):
+        modules = " ".join(modules)
+    XML.SubElement(scm, 'modules').text = modules
+
+    xd = XML.SubElement(scm, 'disableChangeLog')
+    xd.text = str(data.get('disable-changelog', False)).lower()
+
+    browser = data.get('browser', 'auto')
+    browserdict = {
+        'auto': '',
+        'bitbucket': 'BitBucket',
+        'fisheye': 'FishEye',
+        'googlecode': 'GoogleCode',
+        'hgweb': 'HgWeb',
+        'kilnhg': 'KilnHG',
+        'rhodecode': 'RhodeCode',
+        'rhodecode-pre-1.2.0': 'RhodeCodeLegacy'
+    }
+
+    if browser not in browserdict:
+        raise JenkinsJobsException("Browser entered is not valid must be one "
+                                   "of: %s" % ", ".join(browserdict.keys()))
+    if browser != 'auto':
+        bc = XML.SubElement(scm, 'browser', {'class':
+                            'hudson.plugins.mercurial.browser.' +
+                            browserdict[browser]})
+        if 'browser-url' in data:
+            XML.SubElement(bc, 'url').text = data['browser-url']
+        else:
+            raise JenkinsJobsException("A browser-url must be specified along "
+                                       "with browser.")
+
+
 class SCM(jenkins_jobs.modules.base.Base):
     sequence = 30
 
