@@ -22,6 +22,7 @@ Wrappers can alter the way the build is run as well as the build output.
 
 """
 
+import logging
 import xml.etree.ElementTree as XML
 import jenkins_jobs.modules.base
 from jenkins_jobs.errors import JenkinsJobsException
@@ -440,22 +441,29 @@ def port_allocator(parser, xml_parent, data):
     Requires the Jenkins `Port Allocator Plugin.
     <https://wiki.jenkins-ci.org/display/JENKINS/Port+Allocator+Plugin>`_
 
-    :arg str name: Variable name of the port or a specific port number
+    :arg str name: Deprecated, use names instead
+    :arg list names: Variable list of names of the port or list of
+        specific port numbers
 
-    Example::
+    Example:
 
-      wrappers:
-        - port-allocator:
-            name: SERVER_PORT
+    .. literalinclude::  /../../tests/wrappers/fixtures/port-allocator002.yaml
     """
     pa = XML.SubElement(xml_parent,
                         'org.jvnet.hudson.plugins.port__allocator.'
                         'PortAllocator')
     ports = XML.SubElement(pa, 'ports')
-    dpt = XML.SubElement(ports,
-                         'org.jvnet.hudson.plugins.port__allocator.'
-                         'DefaultPortType')
-    XML.SubElement(dpt, 'name').text = data['name']
+    names = data.get('names')
+    if not names:
+        logger = logging.getLogger(__name__)
+        logger.warn('port_allocator name is deprecated, use a names list '
+                    ' instead')
+        names = [data['name']]
+    for name in names:
+        dpt = XML.SubElement(ports,
+                             'org.jvnet.hudson.plugins.port__allocator.'
+                             'DefaultPortType')
+        XML.SubElement(dpt, 'name').text = name
 
 
 def locks(parser, xml_parent, data):
@@ -1018,6 +1026,41 @@ def logstash(parser, xml_parent, data):
         key_sub_element.text = str(redis_config.get('key', 'logstash'))
 
 
+def mongo_db(parser, xml_parent, data):
+    """yaml: mongo-db build wrapper
+    Initalizes a MongoDB database while running the build.
+    Requires the Jenkins `MongoDB plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/MongoDB+Plugin>`_
+
+    :arg str name: The name of the MongoDB install to use
+    :arg str data-directory: Data directory for the server (optional)
+    :arg int port: Port for the server (optional)
+    :arg str startup-params: Startup parameters for the server (optional)
+    :arg int start-timeout: How long to wait for the server to start in
+        milliseconds. 0 means no timeout. (default '0')
+
+    Example:
+
+    .. literalinclude:: /../../tests/wrappers/fixtures/mongo-db001.yaml
+
+    """
+    mongodb = XML.SubElement(xml_parent,
+                             'org.jenkinsci.plugins.mongodb.'
+                             'MongoBuildWrapper')
+    mongodb.set('plugin', 'mongodb')
+
+    if not str(data.get('name', '')):
+        raise JenkinsJobsException('The mongo install name must be specified.')
+    XML.SubElement(mongodb, 'mongodbName').text = str(data.get('name', ''))
+    XML.SubElement(mongodb, 'port').text = str(data.get('port', ''))
+    XML.SubElement(mongodb, 'dbpath').text = str(data.get(
+        'data-directory', ''))
+    XML.SubElement(mongodb, 'parameters').text = str(data.get(
+        'startup-params', ''))
+    XML.SubElement(mongodb, 'startTimeout').text = str(data.get(
+        'start-timeout', '0'))
+
+
 def delivery_pipeline(parser, xml_parent, data):
     """yaml: delivery-pipeline
     If enabled the job will create a version based on the template.
@@ -1053,6 +1096,8 @@ def matrix_tie_parent(parser, xml_parent, data):
     <https://wiki.jenkins-ci.org/display/JENKINS/Matrix+Tie+Parent+Plugin>`_
     Note that from Jenkins version 1.532 this plugin's functionality is
     available under the "advanced" option of the matrix project configuration.
+    You can use the top level ``node`` parameter to control where the parent
+    job is tied in Jenkins 1.532 and higher.
 
     :arg str node: Name of the node.
 
@@ -1173,6 +1218,44 @@ def credentials_binding(parser, xml_parent, data):
             credential_xml.text = params.get('credential-id')
 
 
+def custom_tools(parser, xml_parent, data):
+    """yaml: custom-tools
+    Requires the Jenkins Custom Tools Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Custom+Tools+Plugin>`_
+
+    :arg list tools: List of custom tools to add
+                     (optional)
+    :arg bool skip-master-install: skips the install in top level matrix job
+                                   (default 'false')
+    :arg bool convert-homes-to-upper: Converts the home env vars to uppercase
+                                      (default 'false')
+
+    Example:
+
+    .. literalinclude:: /../../tests/wrappers/fixtures/custom-tools001.yaml
+    """
+    base = 'com.cloudbees.jenkins.plugins.customtools'
+    wrapper = XML.SubElement(xml_parent,
+                             base + ".CustomToolInstallWrapper")
+
+    wrapper_tools = XML.SubElement(wrapper, 'selectedTools')
+    tools = data.get('tools', [])
+    tool_node = base + '.CustomToolInstallWrapper_-SelectedTool'
+    for tool in tools:
+        tool_wrapper = XML.SubElement(wrapper_tools, tool_node)
+        XML.SubElement(tool_wrapper, 'name').text = str(tool)
+
+    opts = XML.SubElement(wrapper,
+                          'multiconfigOptions')
+    skip_install = str(data.get('skip-master-install', 'false'))
+    XML.SubElement(opts,
+                   'skipMasterInstallation').text = skip_install
+
+    convert_home = str(data.get('convert-homes-to-upper', 'false'))
+    XML.SubElement(wrapper,
+                   'convertHomesToUppercase').text = convert_home
+
+
 class Wrappers(jenkins_jobs.modules.base.Base):
     sequence = 80
 
